#!/usr/local/bin/perl
# ------------------------------------------------------------------------ 
# DNS Scanner v1.1           Copyright (C) 1998 Dave Edis <dave@edis.org>
# ------------------------------------------------------------------------ 
# This program does a reverse DNS lookup for each IP in a 256 node subnet
# and optionally attempts to connect to port 80 (web) and return the web
# server software name.
#
# This program is distributed under the terms of the GNU General Public
# License Version 2 as published by the Free Software Foundation.
# For the latest version and licence info go to http://www.edis.org/perl/
# ------------------------------------------------------------------------ 
# v1.0 Created: 29.Dec.97 - Created by Dave Edis <dave@edis.org>
# v1.1 Revised  04.Jan.98 - Released Code

$ver      = "v1.1";
$ver_date = "04.Jan.98";

# ------------------------------------------------------------------------ 
# Variables : Define variables used in program
# ------------------------------------------------------------------------ 

&Get_Input;                               # Read user form input
$cgidir   = ($0=~m#^(.*)(\\|/)#,$1);      # full dir of script
$logfile  = "$cgidir/dnsscan.log";       # logfile
$logging  = 0;     # 1 = on 0 = off       # log scans to log file
$|++;                                     # Turn off buffering

unless ($^O =~ /win/i) {                # alarm isn't supported by winperl
  $timeout   = 1;                       # Connect Timeout after # seconds
  $SIG{ALRM} = sub { $server="Timed Out"; return};
  }

use Socket;
$PROTO = getprotobyname('tcp')||0; # Get TCP Protocol or use default
$SIG{__DIE__} = \&HTML_Error;         # on die show error HTML

# ------------------------------------------------------------------------ 
# Main : Test conditions and give commands
# ------------------------------------------------------------------------ 

  if ($host) { &Scan;      } 
  else       { print &Template("$cgidir/dnsscan_help.html",'html'); }

exit;

# ------------------------------------------------------------------------ 
# Get Input : Reads user form input into variables or hash array
# ------------------------------------------------------------------------ 

sub Get_Input {

  my($name, $value, $pair, @pairs, $buffer);

  if ($ENV{'REQUEST_METHOD'} eq 'POST') { read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'}); }
  if ($ENV{'REQUEST_METHOD'} eq 'GET')  { $buffer = $ENV{'QUERY_STRING'}; }

  @pairs = split(/&/, $buffer);
  foreach $pair (@pairs) {
    ($name, $value) = split(/=/, $pair);
    $value =~ tr/+/ /;
    $value =~ s/%([A-F0-9]{2})/pack("C", hex($1))/egi;
    ${$name} = $value;
    }
  foreach (@ARGV) { ${$_}=1; }
  }

# ------------------------------------------------------------------------ 
# Source : print out source code for the program
# ------------------------------------------------------------------------ 

sub Source {
  print "Content-type: text/plain\n\n";
  open(FILE,$0) || die("Can't open $0");
  foreach (<FILE>) { print; }
  close(FILE);
  }

# ------------------------------------------------------------------------ 
# Scan : Run through a list of ports and try to connect to each one
# ------------------------------------------------------------------------ 

sub Scan {

  if ($host =~ /[a-z]$/) { $paddr = inet_aton("$host.") || die("Couldn't resolve '$host'"); }
  else                   { $paddr = inet_aton("$host") || die("Couldn't resolve '$host'"); }
  ($name,$aliases,$addrtype,$length,@addrs) = gethostbyaddr($paddr,AF_INET);
  ($a,$b,$c,$d)=unpack("C4",$paddr); $ip  = "$a.$b.$c.$d";
  foreach (@addrs) { ($a,$b,$c,$d)=unpack("C4",$_); $ip2 .= "$a.$b.$c.$d "; }

  foreach ('webip','webhost') { if (${"$_"}) { ${"$_"."_checked"} = "checked"; }}
  print &Template("$cgidir/dnsscan.html",'html'); # Print Scan Header

  ($a,$b,$c,$d) = split(/\./,inet_ntoa($paddr));

  &Log("Scanning $host $ip $name $aliases $ip2");

   for $d (0..255) {
     $paddr = pack("C4",$a,$b,$c,$d);
     ($name,$aliases,$addrtype,$length,@addrs) = gethostbyaddr($paddr,AF_INET); 
     for (@addrs) {
       ($w,$x,$y,$z)=unpack("C4",$_);
       unless ("$a.$b.$c.$d" eq "$w.$x.$y.$z") { $aliases .= " $w.$x.$y.$z"; }
       }

     if ($webip || ($webhost && $name)) { $server = &Connect; }
     unless ($bgcolor) { $bgcolor='bgcolor="#003366"' } else { $bgcolor="" }
     &HTML_Scan_IP;
     $server="";
     }

  $exectime = time-$^T; 
  $execmin = int($exectime/60);
  $execsec = $exectime%60;
  print qq|<br><font face="arial" size=2>Scanned 256 IP addresses on $host in $execmin min $execsec seconds</font>|;
  }


# ------------------------------------------------------------------------ 
# Connect : Attempt to connect to a TCP port/service and return 1 or 0
#           usage : &Connect($port);
# ------------------------------------------------------------------------ 

sub Connect {
  if ($timeout) { alarm $timeout; } # Set connect timeout
  socket(SOCK,AF_INET,SOCK_STREAM,$PROTO)       || return qq|No Socket|;
  connect(SOCK,pack "SnA4x8",AF_INET,80,$paddr) || return qq|<font color="#CCCCCC">No Connect</font>|;
  select((select(SOCK),$|++)[0]);
  print SOCK "GET / HTTP/1.0\n\n";
  while (<SOCK>) { if (m/Server: (.*)$/) { $server = $1; last; }}
  close(SOCK);
  unless ($server) { $server = "Unknown"; }

  if ($timeout) { alarm 0; } # cancel any outstanding alarms
  return $server;
  }

# ------------------------------------------------------------------------ 
# Log : Make an entry in the log file if logging is enabled
# ------------------------------------------------------------------------ 

sub Log {

  if ($logging) {

    $date = localtime(time);
    open (FILE,">>$logfile") || die("Can't open $logfile");
    print FILE "$date - $_[0] - $ENV{'REMOTE_HOST'} ($ENV{'REMOTE_ADDR'})\n";
    close(FILE);
    }
}


# ------------------------------------------------------------------------ 
# Error : Display any error messages and exit
#         usage: die("error message", __LINE__); 
# ------------------------------------------------------------------------ 

sub Error {
  print "Content-type: text/html\n\n";
  print "<br>\n Error: $_[0] : $!";
  if ($_[1]) { print " at line $_[1]<br>\n"; }
  exit;
}

# ------------------------------------------------------------------------ 
# HTML : HTML screens used in the program
# ------------------------------------------------------------------------ 

sub HTML_Scan_IP {
  print <<HTML;
<table border=0 width=580 cellspacing=0 cellpadding=0 $bgcolor><tr>
 <td><font face="arial" size=1 color="#FFFFFF"><b>$a.$b.$c.$d : $name $aliases</b></font></td>
 <td valign=top align=right><font face="arial" size=1 color="#FFFFFF"><b>$server</b></font></td>
</tr></table>

HTML
}

sub HTML_Error {
  print "Content-type: text/html\n\n" unless ($ContentType++ > 0);
  print "ERROR: $_[0]";
  exit;
  }

# ---------------------------------------------------------------------------- 
# Template : Open a template, translate special vars, and return contents
#            usage: $var = &Template("filepath");
#                   print &FILE("filepath",'html'); for html content header
#                   print &FILE("filepath",'text'); for text content header
# ---------------------------------------------------------------------------- 

sub Template {  

  local(*FILE);

  if    ($_[1] eq 'html') { print "Content-type: text/html\n\n"  unless ($ContentType++ > 0); }
  elsif ($_[1] eq 'text') { print "Content-type: text/plain\n\n" unless ($ContentType++ > 0); }

  if    (!$_[0])          { return "Template : No file was specified<br>\n"; }
  elsif (-f "$_[0]") {
    open(FILE, "<$_[0]") || return "Template : Couldn't open $_[0]<br>\n";
    while (<FILE>) {
      s/<!-- insert="(.*?)" -->/\1/g;                     # remove <!-- insert="" -->
      s/\$([a-z0-9-_]+)\$/${$1}/gi;                       # matches $scalars$
      s/@([a-z0-9-_]+)@/@{$1}/gi;                         # matches @arrays@
      s/\$([a-z0-9-_]+)\[(\d+?)\]\$/${$1}[$2]/gi;         # matches $array[val]$
      s/\$([a-z0-9-_]+){'([a-z0-9-_]+)'}\$/$1->{$2}/gi;   # matches $hash{'val'}$\
      $FILE .= $_;
      }
    close(FILE);
    } else { return "Template : File '$_[0]' does not exist<br>\n"; }

  return $FILE;
  }

# ------------------------------------------------------------------------ 
#                                 Programming by Dave Edis <dave@edis.org>
